# -*- coding: utf-8 -*-
"""
/***************************************************************************
 RealCentroid
                                 A QGIS plugin
 Create point shape from polygon centroids
                              -------------------
        begin                : 2013-10-27
        copyright            : (C) 2013 by Zoltan Siki
        email                : siki at agt.bme.hu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
# Initialize Qt resources from file resources.py
import resources_rc
# Import the code for the dialog
from realcentroiddialog import RealCentroidDialog
import os.path
import util
#import pdb

class RealCentroid:

    def __init__(self, iface):
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value("locale/userLocale")[0:2]
        localePath = os.path.join(self.plugin_dir, 'i18n', 'realcentroid_{}.qm'.format(locale))

        if os.path.exists(localePath):
            self.translator = QTranslator()
            self.translator.load(localePath)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        # Create the dialog (after translation) and keep reference
        self.dlg = RealCentroidDialog()

    def initGui(self):
        # Create action that will start plugin configuration
        self.action = QAction(
            QIcon(":/plugins/realcentroid/icon.png"),
            u"Real centroids", self.iface.mainWindow())
        # connect the action to the run method
        self.action.triggered.connect(self.run)

        # Add toolbar button and menu item
        self.iface.addToolBarIcon(self.action)
        self.iface.addPluginToMenu(u"&realcnetroid", self.action)

    def unload(self):
        # Remove the plugin menu item and icon
        self.iface.removePluginMenu(u"&realcnetroid", self.action)
        self.iface.removeToolBarIcon(self.action)

    # create centroids shape file
    def centroids(self):
        vlayer = util.getMapLayerByName(self.dlg.ui.layerBox.currentText())
        vprovider = vlayer.dataProvider()
        writer = QgsVectorFileWriter(self.dlg.shapefileName, self.dlg.encoding, vprovider.fields(), QGis.WKBPoint, vprovider.crs())
        inFeat = QgsFeature()
        outFeat = QgsFeature()
        fit = vprovider.getFeatures()
        while fit.nextFeature(inFeat):
            #nElement += 1
            inGeom = inFeat.geometry()
            atMap = inFeat.attributes()
            outGeom = inGeom.centroid()
            if not inGeom.contains(outGeom):
                if inGeom.wkbType() == QGis.WKBPolygon:
                    poly = inGeom.asPolygon()
                elif inGeom.wkbType() == QGis.WKBMultiPolygon:
                    poly = inGeom.asMultiPolygon()
                for part in poly:
                    x1 = part[0][0]
                    y1 = part[0][1]
                    x2 = part[1][0]
                    y2 = part[1][1]
                    for i in range(2, len(part)):
                        x3 = part[i][0]
                        y3 = part[i][1]
                        tmpGeom = QgsGeometry.fromPolygon([[QgsPoint(x1, y1), QgsPoint(x2, y2), QgsPoint(x3, y3)]])
                        outGeom = tmpGeom.centroid()
                        found = inGeom.contains(outGeom)
                        if found:
                           break
                        x1, y1 = x2, y2
                        x2, y2 = x3, y3
                    if found:
                       break
            outFeat.setAttributes(atMap)
            outFeat.setGeometry(outGeom)
            writer.addFeature(outFeat)
        del writer
		# add centroid shape to canvas
        if self.dlg.ui.addBox.checkState() == Qt.Checked:
           if not util.addShape(self.dlg.shapefileName):
               QMessageBox.warning(self, "RealCentroid", "Error loading shapefile:\n" + self.dlg.shapefileName)

    # run method that performs all the real work
    def run(self):
        # show the dialog
        self.dlg.show()
        # Run the dialog event loop
        result = self.dlg.exec_()
        # See if OK was pressed
        if result == 1:
            self.centroids()
